"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.msgLacksPayload = exports.hasSessionDisconnectError = exports.makeSession = exports.makeEncode = exports.makeEmptySessionMessage = exports.decodeWithOptions = exports.decode = exports.readMsg = exports.decodeRecord = exports.decodeId = exports.makeRecipes = void 0;
var util_1 = require("./util");
var resolve_1 = __importDefault(require("./recipes/resolve"));
var operate_1 = __importDefault(require("./recipes/operate"));
var subscribe_1 = __importDefault(require("./recipes/subscribe"));
var _1_0_1_1_1 = __importDefault(require("./1.0,1.1"));
var _1_2_1 = __importDefault(require("./1.2"));
var common_1 = __importDefault(require("./common"));
// Note: version files are empty for now, they used to be necessary but were later made obsolete. I still keep them around in case version specific changes crop up again.
var versionSpecificCommands = {
    "1.0": _1_0_1_1_1.default,
    "1.1": _1_0_1_1_1.default,
    "1.2": _1_2_1.default,
};
var getCommand = function (version, commandType) {
    return version in versionSpecificCommands &&
        commandType in versionSpecificCommands[version]
        ? versionSpecificCommands[version][commandType]
        : common_1.default[commandType] || null;
};
var recipes = [resolve_1.default, operate_1.default, subscribe_1.default];
var makeRecipes = function (call, on) {
    return recipes.reduce(function (acc, _a) {
        var _b;
        var make = _a.make, name = _a.name;
        return (__assign(__assign({}, acc), (_b = {}, _b[name] = make(call, on), _b)));
    }, {});
};
exports.makeRecipes = makeRecipes;
var decodeId = function (data) { return String(data); };
exports.decodeId = decodeId;
var unkownErr = function (msg) { return ["error", "", msg]; };
var decodeRecord = function (proto, data) {
    var record = proto.rootRecord.lookupType("usp_record.Record");
    var decodedRecord = record.decode("binaryData" in data ? data.binaryData : data);
    return JSON.parse(JSON.stringify(decodedRecord));
};
exports.decodeRecord = decodeRecord;
var readMsg = function (proto, data) {
    var _a, _b;
    var record = proto.rootRecord.lookupType("usp_record.Record");
    var decodedRecord = record.decode("binaryData" in data ? data.binaryData : data);
    var convertedDecodedRecord = JSON.parse(JSON.stringify(decodedRecord));
    if ("disconnect" in convertedDecodedRecord)
        return [{}, convertedDecodedRecord.disconnect, true];
    try {
        var msg = proto.rootMsg.lookupType("usp.Msg");
        var decodedMsg = msg.decode(((_a = decodedRecord.noSessionContext) === null || _a === void 0 ? void 0 : _a.payload) ||
            ((_b = decodedRecord.sessionContext) === null || _b === void 0 ? void 0 : _b.payload[0]));
        // forces conversion
        return [JSON.parse(JSON.stringify(decodedMsg)), null, false];
    }
    catch (err) {
        return [{}, err, false];
    }
};
exports.readMsg = readMsg;
var decode = function (parsedMsg, version) {
    var err = util_1.searchParent(parsedMsg, "errMsg") || null;
    var command = util_1.extractCommand(parsedMsg);
    var foundId = util_1.search(parsedMsg, "msgId");
    // if id is formatted by me (command@) then use, otherwise check for sub id
    var id = foundId.includes("@")
        ? foundId
        : util_1.search(parsedMsg, "subscriptionId") || null;
    // if command is unkown
    if (!command)
        return unkownErr(parsedMsg);
    if (err)
        return [id, null, err, command];
    var cmd = getCommand(version, command);
    if (!cmd)
        return unkownErr(parsedMsg);
    var _a = cmd.decode(parsedMsg), decodedData = _a[0], decodedId = _a[1], decodedErr = _a[2];
    return [decodedId || id, decodedData, decodedErr || err, command];
};
exports.decode = decode;
var decodeWithOptions = function (parsedMsg, cmdType, options, version) {
    var cmd = getCommand(version, cmdType);
    if (!cmd)
        return unkownErr(parsedMsg);
    if (options.raw)
        return parsedMsg;
    var decodedData = cmd.decode(parsedMsg, options)[0];
    return decodedData;
};
exports.decodeWithOptions = decodeWithOptions;
var makeEmptySessionMessage = function (proto, bufferOptions, sessionOptions, version, useSession) {
    var converted = _convert(proto, {});
    if (isError(converted))
        return null;
    var encoded = proto.rootMsg
        .lookupType("usp.Msg")
        .encode(converted)
        .finish();
    var buffer = util_1.makeBuffer(proto.rootRecord, encoded, version, bufferOptions, sessionOptions, useSession);
    return buffer;
};
exports.makeEmptySessionMessage = makeEmptySessionMessage;
var makeEncode = function (proto, bufferOptions, useSession) {
    return function (command, version, args, sessionOptions) {
        var cmd = getCommand(version, command);
        if (!cmd)
            return ["error", null, "Uknown command: " + command];
        var msg = cmd.encode(args);
        var id = msg.header.msgId;
        msg.header.msgType = proto.header.MsgType[msg.header.msgType];
        var converted = _convert(proto, msg);
        if (isError(converted))
            return [id, null, converted];
        var encoded = proto.rootMsg
            .lookupType("usp.Msg")
            .encode(converted)
            .finish();
        var buffer = util_1.makeBuffer(proto.rootRecord, encoded, version, bufferOptions, sessionOptions, useSession);
        return [id, buffer, null];
    };
};
exports.makeEncode = makeEncode;
var makeSession = function (sessionId) { return ({
    sessionId: sessionId || 1,
    sequenceId: 1,
    expectedId: 1,
}); };
exports.makeSession = makeSession;
var hasSessionDisconnectError = function (msg) { var _a; return ((_a = msg === null || msg === void 0 ? void 0 : msg.disconnect) === null || _a === void 0 ? void 0 : _a.reasonCode) === 7105; };
exports.hasSessionDisconnectError = hasSessionDisconnectError;
var msgLacksPayload = function (msg) { var _a; return ((_a = msg === null || msg === void 0 ? void 0 : msg.sessionContext) === null || _a === void 0 ? void 0 : _a.payload[0]) === ""; };
exports.msgLacksPayload = msgLacksPayload;
var isError = function (o) { return typeof o == "string"; };
var internalKeys = ["lookup"];
var isInternal = function (key) { return internalKeys.includes(key); };
var makePayload = function (items, isArr) {
    return items
        .filter(function (_a) {
        var k = _a[0];
        return !isInternal(k);
    })
        .reduce(function (acc, _a) {
        var _b;
        var k = _a[0], v = _a[1];
        return isArr
            ? __spreadArray(__spreadArray([], acc), [v]) : __assign(__assign({}, acc), (_b = {}, _b[k] = v, _b));
    }, isArr ? [] : {});
};
var isStringArray = function (obj) {
    return Array.isArray(obj) && obj.every(function (v) { return typeof v === "string"; });
};
var needsConversion = function (v) { return typeof v === "object" && !isStringArray(v); };
var _convert = function (proto, value) {
    var skip = value.lookup === undefined;
    var lookup = "usp." + value.lookup;
    var item = skip ? null : proto.rootMsg.lookupType(lookup);
    var simpleValues = Object.entries(value).filter(function (_a) {
        var v = _a[1];
        return !needsConversion(v);
    });
    var toConvert = Object.entries(value).filter(function (_a) {
        var v = _a[1];
        return needsConversion(v);
    });
    var converted = toConvert.map(function (_a) {
        var k = _a[0], v = _a[1];
        return [
            k,
            _convert(proto, v),
        ];
    });
    var err = converted.find(function (_a) {
        var v = _a[1];
        return isError(v);
    });
    if (err)
        return err[1];
    var total = converted.concat(simpleValues);
    var payload = makePayload(total, Array.isArray(value));
    var payloadErr = item === null || item === void 0 ? void 0 : item.verify(payload);
    if (payloadErr)
        return payloadErr;
    return item ? item.create(payload) : payload;
};
